package fetchKeywords;


import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.store.LockObtainFailedException;
import java.io.IOException;   
import org.apache.lucene.search.IndexSearcher;   
import org.apache.uima.examples.opennlp.annotator.POSTagger;


public class TextFileIndexer {
 
	public int documentsNumber;
    public IndexSearcher searcher; 
    public IndexReader indexReader;
    //fileDir is the directory that contains the text files to be indexed
    File   fileDir  = new File("F:/Master Thesis/input");
  
   //indexDir is the directory that hosts Lucene's index files
    File   indexDir = new File("F:/Master Thesis/luceneIndex");
    File[] textFiles  = fileDir.listFiles();
 
    TextFileIndexer(){
    
	 
	 try {
		searcher= new IndexSearcher("F:/Master Thesis/luceneIndex");
	} catch (CorruptIndexException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	} catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}   
     indexReader=searcher.getIndexReader();
 
 }
 
@SuppressWarnings("deprecation")
   //public static void main(String[] args) throws Exception{
   public void  buildIndex() throws Exception, LockObtainFailedException, IOException{
 
   Analyzer luceneAnalyzer = new StandardAnalyzer();
   IndexWriter indexWriter = new IndexWriter(indexDir,luceneAnalyzer,true);
  
   long startTime = new Date().getTime();
   documentsNumber=textFiles.length;
   //Add documents to the index 
   for(int i = 0; i < textFiles.length; i++){
     if(textFiles[i].isFile() && textFiles[i].getName().endsWith(".txt")){
     
       System.out.println("File " + textFiles[i].getCanonicalPath() 
              + " is being indexed");
       Reader textReader = new FileReader(textFiles[i]);
       Document document = new Document();
	   document.add(new Field("content",textReader));
       document.add(new Field("path",textFiles[i].getPath(),Field.Store.YES,Field.Index.NO));
       indexWriter.addDocument(document);
     }
   }
 
   indexWriter.optimize();
   indexWriter.close();
   }
   /*
   long endTime = new Date().getTime();
   System.out.println("It took " + (endTime - startTime) 
              + " milliseconds to create an index for the files in the directory "
              + fileDir.getPath());
   */
     
    public int docFreq(String queryString) throws Exception, IOException{
        //String queryString = "paris";   
          
        Term term = new Term("content",queryString.toLowerCase());
        		
    	return searcher.docFreq(term);  
       
     }
        	
    public ArrayList<String> docs(String keywords) throws IOException{
    	 
    	  ArrayList<String> docs=new ArrayList<String>();
    	  Term term = new Term("content",keywords.toLowerCase());
          TermDocs termDocs = indexReader.termDocs(term);
         
          while(termDocs.next())
         	{
         		int i=termDocs.doc();
         		docs.add(textFiles[i].getName());
         		
         	}
    	  
           return docs;
         	
    }
    
    public int termFreq(String queryString, int doc) throws CorruptIndexException, IOException{
 
         Term term = new Term("content",queryString.toLowerCase());
         TermDocs termDocs = indexReader.termDocs(term);
        	while(termDocs.next())
        	{
        		if(doc==termDocs.doc())
        			break;
        		//System.out.println("DocNo:   "+termDocs.doc()+"  Freq:   "+termDocs.freq());
        	}
        	return termDocs.freq();
        	
        }
    
    public int termTotal(int doc) throws CorruptIndexException, IOException{
		 int total = 0;
		 TermEnum termEnum=indexReader.terms(); 
	     while(termEnum.next())
	     {
	         total+=termFreq(termEnum.term().text(),doc);      
	     }
		return total;
 	
    }
    
    
    
    public ArrayList<Term> rankingDocFreq() throws IOException{
    	 
    	 System.out.println("---------------------------------------");
    	
         ArrayList<Term> arraylist=new ArrayList<Term>();
         ArrayList<Term> arraylist1=new ArrayList<Term>();
         TermEnum termEnum=indexReader.terms();
         Collection<Integer> docFreq = new  ArrayList<Integer>();
         
         while(termEnum.next()){
        	 
        	String[] posTag=POSTagger.tagger.tag(termEnum.term().text()).split("/");
        //	System.out.println("~~~~~~~~~~~~~~~"+posTag[1]);
            if (posTag[1].equals("NN")||posTag[1].equals("NNS")||posTag[1].equals("NNP"))
            {
            //   System.out.println("{{{{"+termEnum.term().text());
               docFreq.add(termEnum.docFreq());
               arraylist.add(termEnum.term());
            }
        }
         
         Object[] value =  docFreq.toArray() ;
         Arrays.sort(value);
         
         TermEnum termEnum1=indexReader.terms();
        while(termEnum1.next()){
         
         for(int i=0; i<arraylist.size();i++)
         {
         	if(termEnum1.term().equals(arraylist.get(i))){
        	
            Integer integer=(Integer)termEnum1.docFreq();
       
         	if(integer.equals(value[value.length-1])||integer.equals(value[value.length-2])||integer.equals(value[value.length-3])){
                  
         	      arraylist1.add(termEnum1.term());	
         		
               }
         	}
         }
        }
        
        for(int i=0; i<arraylist1.size();i++){    	
     	   
    	   System.out.println("keywords by df:"+arraylist1.get(i).text());
    	}
         return arraylist1;
         
    }
    
    
    
    
    
    
    public ArrayList<Term> tfdf(int document) throws Exception{
   	 
   	  
 
        ArrayList<Term> arraylist=new ArrayList<Term>();
        ArrayList<Term> arraylist1=new ArrayList<Term>();
        TermEnum termEnum=indexReader.terms();
        Collection<Double> tfdf = new  ArrayList<Double>();
        HashMap<Term,Double> hashmap=new HashMap<Term,Double>();
        
        
        while(termEnum.next())
        {
        	TermDocs termDocs = indexReader.termDocs(termEnum.term());
         	while(termDocs.next())
         	{
         		if(document==termDocs.doc()){
         			//break;
         		//System.out.println("DocNo:   "+termDocs.doc()+"  Freq:   "+termDocs.freq());
         	
           String[] posTag=POSTagger.tagger.tag(termEnum.term().text()).split("/");
           if(termEnum.term().text().equals("right"))
               System.out.println(termEnum.term().text()+"~~~~~~~~~~~~~~~"+posTag[1]);
               
               
           if (posTag[1].equals("NN")||posTag[1].equals("NNS")||posTag[1].equals("NNP"))
           {
           //   System.out.println("{{{{"+termEnum.term().text());
        	  double tf= (double)termFreq(termEnum.term().text(),document)/(double)termTotal(document);
        	//  System.out.println(tf);
        	  double df=(double)docFreq(termEnum.term().text())/(double)documentsNumber;
        	  //System.out.println(df);
        	  tfdf.add(tf*df);
              hashmap.put(termEnum.term(),tf*df);
           }
         }
         		break;}
       }
        
        Object[] value =  tfdf.toArray() ;
        Arrays.sort(value);
        
     //   System.out.println(value.length);
     
        Iterator<Term> iterator = (hashmap.keySet()).iterator();
        
        while(iterator.hasNext()) {

        Term key = iterator.next();
        Double values = hashmap.get(key);        	
      	
        if((value.length-1)>=0){
        if(values.equals(value[value.length-1])){        
      	     arraylist1.add(key);	
      		//arraylist1.add(hashmap.get(value[0]));	
      	}
        }
        
        if((value.length-2)>=0){
        if(values.equals(value[value.length-2])){
        	 arraylist1.add(key);	  
        	 //arraylist1.add(hashmap.get(value[1]));	
         }
        }
        
        if((value.length-3)>=0){
         if(values.equals(value[value.length-3])){
        	  arraylist1.add(key);	
        	// arraylist1.add(hashmap.get(value[2])); 
         }
        }
        }
       
       for(int i=0; i<arraylist1.size();i++){    	
    	   
   	       System.out.println("keywords by tfdf:"+arraylist1.get(i).text());
   	  }
       
       return arraylist1;
        
   }
    
    
    
    public ArrayList<Term> df(int document) throws Exception{
      	 
        ArrayList<Term> arraylist=new ArrayList<Term>();
        ArrayList<Term> arraylist1=new ArrayList<Term>();
        TermEnum termEnum=indexReader.terms();
        Collection<Double> dfs = new  ArrayList<Double>();
        HashMap<Term,Double> hashmap=new HashMap<Term,Double>();
        
        
        while(termEnum.next())
        {
        	TermDocs termDocs = indexReader.termDocs(termEnum.term());
         	while(termDocs.next())
         	{
         		if(document==termDocs.doc()){
         			//break;
         		//System.out.println("DocNo:   "+termDocs.doc()+"  Freq:   "+termDocs.freq());
         	
           String[] posTag=POSTagger.tagger.tag(termEnum.term().text()).split("/");
           if(termEnum.term().text().equals("right"))
               System.out.println(termEnum.term().text()+"~~~~~~~~~~~~~~~"+posTag[1]);
               
               
           if (posTag[1].equals("NN")||posTag[1].equals("NNS")||posTag[1].equals("NNP"))
           {
           //   System.out.println("{{{{"+termEnum.term().text());
        	 // double tf= (double)termFreq(termEnum.term().text(),document)/(double)termTotal(document);
        	//  System.out.println(tf);
        	  double df=(double)docFreq(termEnum.term().text())/(double)documentsNumber;
        	  //System.out.println(df);
        	  dfs.add(df);
              hashmap.put(termEnum.term(),df);
           }
         }
         		break;}
       }
        
        Object[] value =  dfs.toArray() ;
        Arrays.sort(value);
        
     //   System.out.println(value.length);
     
        Iterator<Term> iterator = (hashmap.keySet()).iterator();
        
        while(iterator.hasNext()) {

        Term key = iterator.next();
        Double values = hashmap.get(key);        	
      	
        if((value.length-1)>=0){
        if(values.equals(value[value.length-1])){        
      	     arraylist1.add(key);	
      		//arraylist1.add(hashmap.get(value[0]));	
      	}
        }
        
        if((value.length-2)>=0){
        if(values.equals(value[value.length-2])){
        	 arraylist1.add(key);	  
        	 //arraylist1.add(hashmap.get(value[1]));	
         }
        }
        
        if((value.length-3)>=0){
         if(values.equals(value[value.length-3])){
        	  arraylist1.add(key);	
        	// arraylist1.add(hashmap.get(value[2])); 
         }
        }
        }
       
       for(int i=0; i<arraylist1.size();i++){    	
    	   
   	       System.out.println("keywords by tfdf:"+arraylist1.get(i).text());
   	  }
       
       return arraylist1;
        
   }
    
   
	public static void main(String[] args) throws Exception {
    	
    	TextFileIndexer index=new TextFileIndexer();
    	index.buildIndex();
    	System.out.println(index.docFreq("girl"));
    	System.out.println(index.docFreq("girls"));
    	
    }
  
}  



